#include <l4/re/c/dataspace.h>

#include "dataspace_server.h"
#include "memory_incremental.h"
#include "ipc.h"
#include "simple_pager.h"



SimplePager::SimplePager(Memory *memory)
{
    if (memory == NULL)
        _memory = new MemoryIncremental();
    else
        _memory = memory;

    _region = _memory->region();
}

void SimplePager::close()
{
    if (_region != NULL)
    {
        _memory->release(_region);
        _region = NULL;
    }
}

/* Map a flexpage corresponding to the dataspace 'offset' involving a 'hot_spot'
   (flexpage offset). */

long SimplePager::map(offset_t offset, address_t hot_spot, flags_t flags, l4_snd_fpage_t *region)
{
    Flexpage flexpage(_region);

    flexpage.reset(offset);
    flexpage.upgrade(flags);

    /* Send the flexpage explicitly. */

    long err = ipc_prepare_flexpage(&flexpage, offset, _region->size(),
                                    hot_spot, flags, region);

    if (err)
        return err;

    err = complete_Dataspace_map(*region);

    if (err)
        return err;

    return IPC_MESSAGE_SENT;
}

long SimplePager::info(offset_t *size, flags_t *flags)
{
    *size = _region->size();
    *flags = L4_FPAGE_RW;

    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
